@@ -1,6 +1,4 @@
 /*
- *  xnec2c - GTK2-based version of nec2c, the C translation of NEC2
- *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -16,11 +14,10 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#ifndef	DRAW_STRUCTURE_H
-#define	DRAW_STRUCTURE_H		1
+#ifndef DRAW_STRUCTURE_H
+#define DRAW_STRUCTURE_H        1
 
 #include "common.h"
-#include "support.h"
 
 #endif
 
